@@ -4,9 +4,9 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(PE_OK);
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.12';
 
-extends 'Lemonldap::NG::Portal::Auth::Base',
+extends 'Lemonldap::NG::Portal::Main::Auth',
   'Lemonldap::NG::Portal::Lib::Remote';
 
 # RUNNING METHODS
@@ -16,13 +16,13 @@ sub extractFormInfo {
     my $r = $self->checkRemoteId($req);
     return $r unless ( $r == PE_OK );
     $req->{user} =
-      $req->datas->{rSessionInfo}->{ $self->conf->{remoteUserField} || 'uid' };
-    $req->datas->{password} = $req->datas->{rSessionInfo}->{'_password'};
-    PE_OK;
+      $req->data->{rSessionInfo}->{ $self->conf->{remoteUserField} || 'uid' };
+    $req->data->{password} = $req->data->{rSessionInfo}->{'_password'};
+    return PE_OK;
 }
 
 sub authenticate {
-    PE_OK;
+    return PE_OK;
 }
 
 sub setAuthSessionInfo {
@@ -30,14 +30,14 @@ sub setAuthSessionInfo {
 
     # Store password (deleted in checkRemoteId() if local policy does not accept
     # stored passwords)
-    $req->{sessionInfo}->{'_password'} = $req->datas->{'password'};
+    $req->{sessionInfo}->{'_password'} = $req->data->{'password'};
     $req->{sessionInfo}->{authenticationLevel} =
-      $req->datas->{rSessionInfo}->{authenticationLevel};
-    PE_OK;
+      $req->data->{rSessionInfo}->{authenticationLevel};
+    return PE_OK;
 }
 
 sub authLogout {
-    PE_OK;
+    return PE_OK;
 }
 
 sub getDisplayType {
